;todo: verify PROCs

; exports 
;dependencies
        EXPORT  GetChar
        EXPORT  GetStringSB
        EXPORT  Init_UART0_IRQ
        EXPORT  PutChar
        EXPORT  PutNumHex
        EXPORT  PutNumUB
        EXPORT  PutStringSB
        EXPORT  UART0_IRQHandler


;>>>>>> subroutine dependencies <<<<<<<<<<
		EXPORT 	Init_PIT_IRQ
Init_PIT_IRQ         PROC        {R1-R14} 
; Description: 
;   Only run on the first PIT Interrupt.
;   Initialize the PIT to generate an interrupt every 0.01 seconds.
; Inputs:
;   None
; Outputs:
;   None
; Registers modified:
;   None  
; Intermediate registers;
;   

	PUSH 	{R0-R2,LR}

    ;Enable clock for PIT module
    LDR   R0,=SIM_SCGC6             
    LDR   R1,=SIM_SCGC6_PIT_MASK             
    LDR   R2,[R0,#0]             
    ORRS  R2,R2,R1             
    STR   R2,[R0,#0]

    ;Disable PIT timer 0 
    LDR   R0,=PIT_CH0_BASE 
    LDR   R1,=PIT_TCTRL_TEN_MASK 
    LDR   R2,[R0,#PIT_TCTRL_OFFSET] 
    BICS  R2,R2,R1 
    STR   R2,[R0,#PIT_TCTRL_OFFSET]

    ;Set PIT interrupt priority  
    LDR     R0,=PIT_IPR 
    LDR     R1,=NVIC_IPR_PIT_MASK 
    ;LDR     R2,=NVIC_IPR_PIT_PR0_0 
    LDR     R1,[R0,#0] 
    BICS    R1,R1,R1 
    ;ORRS    Rl,Rl,R2 
    STR     R1,[R0,#0] 

    ;Clear any pending PIT interrupts 
    LDR     R0,=NVIC_ICPR 
    LDR     R1,=NVIC_ICPR_PIT_MASK 
    STR     R1,[R0,#0]


    ;Unmask PIT interrupts 
    LDR     R0,=NVIC_ISER 
    LDR     R1,=NVIC_ISER_PIT_MASK 
    STR     R1,[R0,#0]
    
    ;Enable PIT module 
    LDR   R0,=PIT_BASE 
    LDR   R1,=PIT_MCR_EN_FRZ 
    STR   R1,[R0,#PIT_MCR_OFFSET] 

    ;Set PIT timer 0 period for 0.01 s 
    LDR   R0,=PIT_CH0_BASE 
    LDR   R1,=PIT_LDVAL_10ms 
    STR   R1,[R0,#PIT_LDVAL_OFFSET]
     
    ;Enable PIT timer 0 interrupt 
    LDR   R1,=PIT_TCTRL_CH_IE 
    STR   R1,[R0,#PIT_TCTRL_OFFSET]

    ; set run stop watch to zero 
    LDR     R0,=RunStopWatch
    MOVS    R1,#0
    STRB    R1,[R0,#0]                      ; set RunStopWatch t

	POP 	{R0-R2,PC}  
    ENDP 


	EXPORT PIT_IRQHandler
PIT_IRQHandler  ;todo: verify that this alias is correct. source: startup_MKL46Z4.s ln 89
	EXPORT 	PIT_ISR
PIT_ISR         PROC        {R0-R14} 
; Description: 
;       On a PIT interrupt, if the (byte) variable RunStopWatch is not zero, 
;       PIT_ISR increments the (word) variable Count; otherwise, it leaves Count unchanged.  
;       In either case, make sure the ISR clears the interrupt condition before exiting. 
;       You must write the ISR so that no registers have changed value after return.
; Inputs:
;     None
; Outputs:
;    None  
; Registers modified:
;    None    
; Intermediate registers;
;    R0-R2
    CPSID   I 
	PUSH 	{LR}
    LDR     R0,=RunStopWatch 
    LDRB    R0,[R0,#0]              ; todo: verify this offset is correct
    CMP     R0,#0
    ;if (RunStopWatch) { check for not zero 
    BEQ     PIT_ISR_Else            ; In either case, make sure the ISR clears the interrupt condition before exiting.
PIT_ISR_Increment_Count             ;    Increment Count 

    LDR     R1,=Count
    LDR     R2,[R1,#0]
    ADDS    R2,R2,#1
    STR     R2,[R1,#0]

PIT_ISR_Else                        ;clear interrupt by writing 1 to T FLG, causing the interrupt to be cleared (TIF Bit is zero)
    LDR     R1,=PIT_CH0_BASE
    LDR     R2,=PIT_TFLG_TIF_MASK
    STR     R2,[R1,#PIT_TFLG_OFFSET]
	
	CPSIE   I
	POP 	{PC}

    ENDP 


GetChar         PROC        {R1-R14} 
; char GetChar (void)
; Description:
;   Dequeues a character from the receive queue, and returns it in R0 
; Inputs:
;   None
; Outputs:
;   R0 the register froDequeuem the keyboard
; 	 
;   APSR C Bit clear =  success
;    APSR C Bit set = Dequeue Failure
; Registers modified:
;   None

	PUSH  {R1,LR}
GetChar_Loop   
	LDR     R1,=RXQueueRecordStructure
	CPSID 	I 		; critical code, so disable interrupts 
    BL      Dequeue     ; char received now in R0
	CPSIE	I 		; unmask interrupts 
	BCS 	GetChar_Loop
	POP		{R1,PC}
	ENDP

;****************************************************************
GetStringSB         PROC                {R0-R14}
; General Function:
    ;Uses GetChar to get the next char from the keyboard input and puts that char into a string starting at R0,
        ;and uses putchar to echo the character to the termail
; Special Cases
    ; for n characters, any characters taken in after n-1 will be taken in by getchar, and ignored.
    ; when carage return is detected, return

;inputs
;R0 - starting address of the string 
;R1  - how many chars it takes to cause overflow  
;intermediates 
;R0 -  address (Pointer) of next char
;R1 - Number of chars left until overflow (buffer Capacity)
;R2 -  temp char 
;R3  - zero
;outputs 
;R0 - starting address of the string 


        PUSH        {R0-R3, LR}
        MOVS        R3,#0
GetStringSB_GetNextChar
; Get the next char using GetChar
        PUSH        {R0}                ; Save R0
        BL          GetChar             ; R0 now has the next char
        MOVS        R2,R0               ; R2 and R0 now have the temp char
        SUBS        R1,R1,#1            ; Decrement number of chars until overflow
        BL			PutChar              ;sends r0 to terminal
        POP         {R0}                ; restore r0, starting address of string
; temp char is in R2
; Check special Cases and branch if they are met 
        ; check to see if its an enter 
        CMP         R2,#13
        BEQ         GetStringSB_ReturnDetected
        ; Check to see if there is overflow 
        CMP         R1,#1
        BEQ         GetStringSB_Overflow       ; if we are already over the overflow limit, just look for the next char don't bother saving anything. we are looking for an enter



; Save to string and increment pointer of R0 

        STRB    R2,[R0,#0]
        ADDS    R0,R0,#1
        B       GetStringSB_GetNextChar     ;continue the cycle


GetStringSB_ReturnDetected
        STRB    R3,[R0,#0]		;null terminate string.
        POP    {R0-R3, PC}		;restore registers and branch back
           
;flow will never naturally come here 	   
GetStringSB_Overflow	;overflow means no storage, just looking for enter 	
	    STRB    R2,[R0,#0]			; store last char without echo (thats why we cant loop again)
	    ADDS    R0,R0,#1
GetStringSB_Overflow_Loop
	    PUSH        {R0}                ; Save R0
        BL          GetChar             ; R0 now has the next char
        MOVS        R2,R0               ; R2 and R0 now have the temp char
        POP         {R0}                ; restore r0, starting address of string
	    CMP         R2,#13
        BEQ         GetStringSB_ReturnDetected
	    B			GetStringSB_Overflow_Loop ;loop until enter 
	    ENDP
;****************************************************************
Init_UART0_IRQ         PROC        {R1-R14} 
; Description: 
; Interrupt request (IRQ) line Analogue of BL for subroutine
;       
; Inputs:
;       
; Outputs:
;       
; Registers modified:
;       
; Intermediate registers;
;       
	PUSH 	{R0-R2,LR}


	;Initialize any data structures associated with handling UART0 interrupts:
	;rx queue 
	LDR 	R0,=RXQueueBuffer
	LDR 	R1,=RXQueueRecordStructure
	MOVS 	R2,#UARTQueueBufferSize
	BL 		InitQueue

	;tx queue 

	LDR 	R0,=TXQueueBuffer
	LDR 	R1,=TXQueueRecordStructure
	;MOVS 	R2,=UARTQueueBufferSize ; already done above
	BL 		InitQueue

	;Dr. Melton's code:
	;Set UART0 IRQ priority
	LDR     R0,=UART0_IPR
	;LDR     R1,=NVIC_IPR_UART0_MASK
	LDR     R2,=NVIC_IPR_UART0_PRI_3
	LDR     R3,[R0,#0]
	;BICS    R3,R3,R1
	ORRS    R3,R3,R2
	STR     R3,[R0,#0]
	;Clear any pending UART0 interrupts
	LDR     R0,=NVIC_ICPR
	LDR     R1,=NVIC_ICPR_UART0_MASK
	STR     R1,[R0,#0]
	;Unmask UART0 interrupts
	LDR     R0,=NVIC_ISER 
	LDR     R1,=NVIC_ISER_UART0_MASK
	STR     R1,[R0,#0]



;Select MCGPLLCLK / 2 as UART0 clock source
    LDR   R0,=SIM_SOPT2
    LDR   R1,=SIM_SOPT2_UART0SRC_MASK
    LDR   R2,[R0,#0]
    BICS  R2,R2,R1
    LDR   R1,=SIM_SOPT2_UART0_MCGPLLCLK_DIV2
    ORRS  R2,R2,R1
    STR   R2,[R0,#0]
;Enable external connection for UART0
	LDR R0,=SIM_SOPT5
	LDR R1,=SIM_SOPT5_UART0_EXTERN_MASK_CLEAR
	LDR R2,[R0,#0]
	BICS R2,R2,R1
	STR R2,[R0,#0]
	;Enable clock for UART0 module
	LDR R0,=SIM_SCGC4
	LDR R1,=SIM_SCGC4_UART0_MASK
	LDR R2,[R0,#0]
	ORRS R2,R2,R1
	STR R2,[R0,#0]
;Enable clock for Port A module
	LDR R0,=SIM_SCGC5
	LDR R1,=SIM_SCGC5_PORTA_MASK
	LDR R2,[R0,#0]
    ORRS  R2,R2,R1
    STR   R2,[R0,#0]
;Connect PORT A Pin 1 (PTA1) to UART0 Rx (J1 Pin 02)
    LDR     R0,=PORTA_PCR1
    LDR     R1,=PORT_PCR_SET_PTA1_UART0_RX
    STR     R1,[R0,#0]
;Connect PORT A Pin 2 (PTA2) to UART0 Tx (J1 Pin 04)
    LDR     R0,=PORTA_PCR2
    LDR     R1,=PORT_PCR_SET_PTA2_UART0_TX
    STR     R1,[R0,#0]
;Disable UART0 receiver and transmitter
    LDR   R0,=UART0_BASE
    MOVS  R1,#UART0_C2_T_R
    LDRB  R2,[R0,#UART0_C2_OFFSET]
    BICS  R2,R2,R1
    STRB  R2,[R0,#UART0_C2_OFFSET]
;Set UART0 for 9600 baud, 8N1 protocol
    MOVS  R1,#UART0_BDH_9600
    STRB  R1,[R0,#UART0_BDH_OFFSET]
    MOVS  R1,#UART0_BDL_9600
    STRB  R1,[R0,#UART0_BDL_OFFSET]
    MOVS  R1,#UART0_C1_8N1
    STRB  R1,[R0,#UART0_C1_OFFSET]
    MOVS  R1,#UART0_C3_NO_TXINV
    STRB  R1,[R0,#UART0_C3_OFFSET]
    MOVS  R1,#UART0_C4_NO_MATCH_OSR_16
    STRB  R1,[R0,#UART0_C4_OFFSET]
    MOVS  R1,#UART0_C5_NO_DMA_SSR_SYNC
    STRB  R1,[R0,#UART0_C5_OFFSET]
    MOVS  R1,#UART0_S1_CLEAR_FLAGS
    STRB  R1,[R0,#UART0_S1_OFFSET]
    MOVS  R1, \
	    #UART0_S2_NO_RXINV_BRK10_NO_LBKDETECT_CLEAR_FLAGS
    STRB  R1,[R0,#UART0_S2_OFFSET]
;Enable UART0 receiver and transmitter
	MOVS  R1,#UART0_C2_T_RI						
    STRB  R1,[R0,#UART0_C2_OFFSET]

	POP 	{R0-R2,PC}
	ENDP

;********************************************************************
; dependencies: ------------------------------------------------------
UART0_IRQHandler
UART0_ISR         PROC        {R0-R14} 
; Description: 
; 		UART0 Interrupt Service Routine 
;       handles UART0 transmit and receive interrupts
; 		Enqueues characters received by UART to RxQ – 
;		Dequeues characters from TxQ for UART to send

; Inputs:
;       
; Outputs:
;       
; Registers modified:
;       
; Intermediate registers;
;   
	CPSID 	I 		
	PUSH 	{LR}
	;Interrupt source can be found in UART0_S1 

; UART 0 Register maps 
;UART0_C2
;0-->7:TIE=transmit IE for TDRE (disabled)
;0-->6:TCIE=transmission complete IE for TC (disabled)
;0-->5:RIE=receiver IE for RDRF (disabled)
;0-->4:ILIE=idle line IE for IDLE (disabled)
;1-->3:TE=transmitter enable (enabled)
;1-->2:RE=receiver enable (enabled)
;0-->1:RWU=receiver wakeup control (normal)
;0-->0:SBK=send break (disabled, normal)


;UART0_S1
;0-->7:TDRE=transmit data register empty flag; read-only
;0-->6:TC=transmission complete flag; read-only
;0-->5:RDRF=receive data register full flag; read-only
;1-->4:IDLE=idle line flag; write 1 to clear (clear)
;1-->3:OR=receiver overrun flag; write 1 to clear (clear)
;1-->2:NF=noise flag; write 1 to clear (clear)
;1-->1:FE=framing error flag; write 1 to clear (clear)
;1-->0:PF=parity error flag; write 1 to clear (clear)

	; TX Side ISR -------------

UART0_ISR_Tx
    LDR   	R0,=UART0_BASE	
	; check if tx interupt is enabled. 	
    MOVS  	R1,#UART0_C2_TE_MASK			
    LDRB  	R2,[R0,#UART0_C2_OFFSET]		; R2 has the UART 0 C2 Register 
	ANDS  	R2,R2,R1						; R2 now has only the transmit enable bit in UART0_C2 (bit 3 on lecture 10 pg 7)
	BEQ 	UART0_ISR_Rx					; if (TxInteruptEnabled) continue, else branch UART0_ISR_Rx. since r2=0, ;TIE = 0 in UART0_C2, so go to the rx side.
	; check if tx interupt has actually occurred, if it hasn't go to rx. 
	MOVS  	R1,#UART0_S1_TDRE_MASK			
	LDRB  	R2,[R0,#UART0_S1_OFFSET]		; R2 has the UART 0 S1 Register 
	ANDS  	R2,R2,R1						; R2 now has only the UART0_S1_TDRE bit (7)
	BEQ 	UART0_ISR_Rx					; if (TxInterupt) continue, else branch UART0_ISR_Rx. since r2=0, ;TDRE = 0 in UART0_S1, so go to the rx side.
		; if we are here, both tx is enabled and the tx has actually occurred 
	;dequeue char ; Input = R1:	address of record structure
	LDR 	R1,=TXQueueRecordStructure		;Dequeue character from TxQueue
	BL 		Dequeue							; data stored in r0. WARNING R0 CHANGED 
	BCS 	UART0_ISR_Tx_DisableTxInterrupt	; if (dequeue successful) then continue, else B UART0_ISR_Tx_DisableTxInterrupt
		;Write character to UART0_D ;Tx data reg. 
    LDR   	R1,=UART0_BASE
    STRB 	 R0,[R1,#UART0_D_OFFSET]
	B		UART0_ISR_Rx
UART0_ISR_Tx_DisableTxInterrupt												
	;Disable UART0 transmitter interupt 
    LDR   	R0,=UART0_BASE	
	MOVS  	R1,#UART0_C2_TIE_MASK
    LDRB  	R2,[R0,#UART0_C2_OFFSET]
    BICS  	R2,R2,R1						; force UART0_C2_T_RI to zero
    STRB  	R2,[R0,#UART0_C2_OFFSET]
	;B		UART0_ISR_Rx


	; RX Side ISR -------------
UART0_ISR_Rx
	; check if rx interupt has actually occurred, if it hasn't end
    LDR   	R0,=UART0_BASE	
	MOVS  	R1,#UART0_S1_RDRF_MASK
    LDRB  	R2,[R0,#UART0_S1_OFFSET]		; R2 has the UART 0 C2 Register 
	ANDS  	R2,R2,R1						; R2 now has only the RDRF bit 
	BEQ		UART0_ISR_End					; if rx interupt is disabled (RDRF = 0) branch to UART0_ISR_End
	; rx interupt happened. read data and enqueue 
	LDRB  	R0,[R0,#UART0_D_OFFSET] 	; read data ; R0: 	char to enqueue 
	LDR 	R1,=RXQueueRecordStructure	; R1:	address of record structure
	BL 		Enqueue						; enqueue 
UART0_ISR_End
	CPSIE 	I 
	POP 	{PC}    
	ENDP 


InitQueue	PROC	{R0-R13}
; Description:
;       Initializes the queue record structure
; Inputs:
; R0:	address of queue buffer
; R1:	address to initializes the queue record structure at
; R2:	size of buffer
; Outputs:
; R1: 	queue record structure (via reference by unsigned word address) 

; Registers modified:
;       NONE
; Intermediate registers;
;       R3 : #0
;	R4 : address passed the queue buffer = R0 +R2
	PUSH 	{R1, R3-R4, LR}
; when the queue is initialized (empty) the inPointer and bufferStart are the same
	;Set Intermediate values
	MOVS	R3,#0					;put zero in r3		
	ADDS	R4,R0,R2

	STR		R0,[R1,#QueueMGMTRecord_BUF_STRT]	;set the start address in mgmt record structure to starting address of queue buffer

	STR		R0,[R1,#QueueMGMTRecord_IN_PTR]		;set the in pointer in mgmt record structure to starting address of queue buffer
	STR		R0,[R1,#QueueMGMTRecord_OUT_PTR]		; set the QueueMGMTRecord_OUT_PTR to the address of the last item in the queue

	STR		R4,[R1,#QueueMGMTRecord_BUF_PAST]	; set buff past to address passed the queue buffer, which is R0 +R2

	STRB	R2,[R1,#QueueMGMTRecord_BUF_SIZE]	; set the buffer size in the mgmt record to the buffer size input (r2)

	STRB	R3,[R1,#QueueMGMTRecord_NUM_ENQD]	; set the num enqueued to zero


	POP 	{R1, R3-R4, PC}				; this pop takes care of r1 (output)
	ENDP
;--------------------------------

Dequeue		PROC	{R1-R13}
; Description:
;       Attempts to get a character from the specified queue
;	checks queue size to be greater than zero to make sure there is a char to dequeued
; Inputs:
; R1:	address of record structure
; Outputs:
; R0: 	Char dequeued
; R1: 	queue record structure (via reference by unsigned word address) 
;      	PSR C Bit clear = success
;      	PSR C Bit set = Failure
; 		QueueMGMTRecord_OUT_PTR
;       QueueMGMTRecord_NUM_ENQD
; Registers modified:
;       R0
;      	PSR C Bit
; Intermediate registers;
; R2:	address of item to dequeue, then next outpointer
; R3:	number of elements in the queue
; R4:   buffer past
; R5: 	IN_PTR
	PUSH 	{R1-R5, LR}
	LDR		R4,[R1,#QueueMGMTRecord_BUF_PAST]			; R4:   buffer past

	LDRB	R3,[R1,#QueueMGMTRecord_NUM_ENQD]		; Load the value of the number of elements in the queue to R3

	CMP		R3,#0
	BEQ		Dequeue_QueueEmpty

Dequeue_Success
	; there is more than 1 item in the queue, success
	LDR		R2,[R1,#QueueMGMTRecord_OUT_PTR]		;get address of item to dequeue (R2) from record structure R1, QueueMGMTRecord_OUT_PTR
	LDRB	R0,[R2,#0]								; load the value of the item in OUT_PTR (CHAR, so bytes) to R0
	; increment QueueMGMTRecord_OUT_PTR since its a fifo queue
	ADDS	R2,#1									; increment outpointer as we have taken out an item (char)
	CMP  	R2,R4									;check special case for queue full ; if outpointer = buffer past (outside of valid range), set it to start
	BEQ		Dequeue_FullBuffer						; branch to update outpointer to the start only if the buffer = full
	STR		R2,[R1,#QueueMGMTRecord_OUT_PTR]		; no circular, so store incremented outpointer
Dequeue_UpdateRecord_NUM_ENQD
	SUBS	R3,R3,#1
	STRB	R3,[R1,#QueueMGMTRecord_NUM_ENQD]		; decrement num_enqd
	ClearCFlag										;Macro to clear c flag since dequeue is successfull
	B 		Dequeue_end

Dequeue_QueueEmpty
	;there is no item in the queue, failure
	SetCFlag					;Macro to set c flag
	B	Dequeue_end


Dequeue_end
	POP	{R1-R5, PC}

Dequeue_FullBuffer
	LDR	R5,[R1,#QueueMGMTRecord_BUF_STRT]			; load buffer start to r5
	STR	R5,[R1,#QueueMGMTRecord_OUT_PTR]				; set outpointer to start to keep circular flow and prevent outpointer from going out of range
	B	Dequeue_UpdateRecord_NUM_ENQD
	ENDP
;--------------------------------

Enqueue		PROC	{R0-R13}
; Description: Attempts to put a character in the queue
;
; Inputs:
; R0: 	char to enqueue
; R1:	address of record structure
; Outputs:
; R1: 	queue record structure (via reference by unsigned word address) 
;      	PSR C Bit clear = success
;      	PSR C Bit set = Failure
; Registers modified:
;       R0
;      	PSR C Bit
; Intermediate registers;
;       R3	queue/buffer size
; 	R4	buffer NUM_ENQD
;	R5 	address to enqueue to (IN_PTR)
;	R6	address of BUF_PAST
	PUSH 	{R1-R6, LR}
	;load intermediates
	LDRB	R3,[R1,#QueueMGMTRecord_BUF_SIZE]		; load the buff size in r3
	LDRB	R4,[R1,#QueueMGMTRecord_NUM_ENQD]		; load the num enqueued ( before this enqueue) into r4

	; check if queue is full (Buffer size - NumENQUED is less than zero)
	CMP		R3,R4
	BEQ		Enqueue_QueueFull
Enqueue_QueueSuccess
	ClearCFlag			;queue not full, clear the c flag
	LDR		R5,[R1,#QueueMGMTRecord_IN_PTR]		; put the address where the next item should be queued in (enqueued) in r5
	STRB	R0,[R5,#0]				; store the char (byte) to enqueue (r0) in the address where the next item should be enqueued (r5)
Enqueue_UpdateRecord
	ADDS	R4,R4,#1 		;increment num enqueued copy
	STRB	R4,[R1,#QueueMGMTRecord_NUM_ENQD]	; store num enqueued
	ADDS 	R5,R5,#1		; increment qin
	; check if q in = q past (needing wrap around)
	LDR		R6,[R1,#QueueMGMTRecord_BUF_PAST]
	CMP		R5,R6			; compare IN_PTR (R5) to Buffer Past BUF_PAST
	BEQ 	Enqueue_Circular	;queue needs wrap around
	;queue does not need wrap around
	STR		R5,[R1,#QueueMGMTRecord_IN_PTR]	; no circular, store increment IN_PTR
	B 		Enqueue_END
Enqueue_QueueFull
	SetCFlag
	;B 		Enqueue_END     not needed due to natural flow
Enqueue_END
	POP		{R1-R6, PC}

Enqueue_Circular		; ref lecture 16b pg 13
	; set qin (IN_PTR) to BUF_STRT
	LDR	R5,[R1,#QueueMGMTRecord_BUF_STRT]		; load buff start into the register used for in ptr
	STR	R5,[R1,#QueueMGMTRecord_IN_PTR]			; set in ptr to buff start, circular opperation
	ClearCFlag									; the circular opperation sets C flag, so we must clear it.
	B 	Enqueue_END
	ENDP
;-------------------------------- 

;-------------------------------------------------------------------
PutChar         PROC        {R0-R14} 
; Description:
;   Enqueues the character from R0 to the transmit queue.
; Inputs:
;   R0 - data to be put to transmit queue 
;Outputs:
;   APSR C Bit clear = Enqueue success
;   APSR C Bit set = Enqueue Failure
; Registers modified:
;   None
; todo:  PutChar needs to enable Tx interrupts 
	; ISR has disabled transmit interrupt because the uart very often does not have data to sent to terminal ATM, so we would contently be interrupted
	PUSH    {R0-R1,LR}
	LDR     R1,=TXQueueRecordStructure
PutChar_Loop	 
    CPSID 	I 		; critical code, so disable interrupts 
	BL      Enqueue    
	CPSIE	I 		; unmask interrupts (not disable)
	BCS 	PutChar_Loop
	LDR 	R0,=UART0_BASE
	MOVS 	R1,#UART0_C2_TI_RI			
	STRB 	R1,[R0,#UART_C2_OFFSET]					
	POP		{R0-R1,PC}
	ENDP
;********************************************************************


PutNumHex		PROC	{R0-R13}
; Description:
;       Prints to the terminal screen the text hexadecimal representation of the entire unsigned word value in R0.
; Inputs:
; R0:	unsigned byte value to print in HEX //todo: correct this. i think should should say unsigned word
; Outputs:
;	None
; Registers modified:
;
; Intermediate registers;
; R0:  	the values to be printed next, starting with LSNibble. After printing the nibble is removed
; R1: 	the hex digit (nibble) to be printed, after check digit it is the ascii value of that char
; R3: 	Mask to get the LSNibble via and
; R4: 	loop counter, represents how many hex digits have been printed



	PUSH 	{R0-R4,LR}
	MOVS	R4,#0

PutNumHex_NextNibble
	LDR		R3,=MSNibbleMask	; load 0x_F0000000 to R3
	ANDS	R3,R3,R0			; R3 gets the MSNibble from R0
	MOVS	R1,R3				; R1 gets the MSNibble from R0
	LSRS 	R1,#28				; sift the hex char into LSb

	LSLS	R0,R0,#4			; shift r0 LEFT 4 because the LEFT most NIBBLE has already been taken into r1

PutNumHex_CheckDigit
	; the hex digit could be between 0 and 9, or A and F
	CMP		R1,#0x9				; compare the current hex digit to  0x_9
	BGT		PutNumHex_HexDigitLetter	; if the hex digit is greater than 0x_9, its a letter, go to letter. otherwise its a number, naturally contune to PutNumHex_HexDigitNumber

PutNumHex_HexDigitNumber
	ADDS	R1,R1,#48		;add the ascii offset for numbers '0'=48
	PUSH	{R0}
	MOVS	R0,R1			; Move the ascii representation of the current hex digit into R0 for putchar
	BL		PutChar			; print R0, the ascii representation of the current nibble
	POP		{R0}
	ADDS	R4,#1				; increment number of hex digits printed
	CMP		R4,#8
	BEQ		PutNumHex_End
	B		PutNumHex_NextNibble

PutNumHex_HexDigitLetter
	;ASCII VALUES: 	0x_X + 0x_31 = 'X'
	; 0x_A  = 10
	; 'A' = 41
	ADDS	R1,R1,#55			; letter offset for ascii values
	PUSH	{R0}
	MOVS 	R0,R1
	BL		PutChar				; print R0, the ascii representation of the current nibble
	POP		{R0}
	ADDS	R4,#1				; increment number of hex digits printed
	CMP		R4,#8
	BEQ		PutNumHex_End
	B		PutNumHex_NextNibble
;todo: fix the lack of looping
PutNumHex_End
	POP		{R0-R4,PC}


	ENDP

;********************************************************************



PutNumUB		PROC	{R0-R14}
; Description:
;       send the lest significant byte of R0 to terminal as decimal
; Inputs:
; R0:	unsigned byte value to print in decimal
; Outputs:
;       NONE
; Registers modified:
;       NONE
; Intermediate registers;
; R1: 	Mask to get rid of all but LSByte

	
	PUSH 	{R0-R1,LR}
	LDR		R1,=LSByteMask		;mask off everthing but the lest sig byte
	ANDS	R0,R0,R1		; and with the mask to get rid of all but LSByte
	BL		PutNumU			; print what is left of r0 in decimal
	POP		{R0-R1,PC}
	ENDP
;--------------------------------
;********************************************************************



PutStringSB         PROC                {R0-R14}
; General Function: 
;	Display each char of a string with starting 
;	address r0 until a null char is detected, or R1 (overflow) is reached, 
	; whichever comes earllier
; Special Cases

;inputs
;R0 starting address 
;R1 is the number of chars it takes to cause overflow

;intermediates 
;R2 temp char

;outputs 
;None

	PUSH	{R0-R2,LR}
	
PutStringSB_PutNextChar
	LDRB  	R2,[R0,#0]

PutStringSB_Checks 
    ;check for overflow, decrement chars till overflow
	CMP	R1,#1		; compare chars until overflow with 0
	BEQ 	PutStringSB_End	; if it's equal to, branch to end as the string is full
	SUBS	R1,#1           ; decrement number of chars till error
    ;check for null termination 
	CMP	R2,#0		; compare the temp char with null
	BEQ	PutStringSB_End	; if we have reached the end of the string due to null terminations, branch to the end	
;if we get here all the checks have been passed, so print the char
    PUSH  	{R0}
    MOVS  	R0,R2                       ; move temp char to r0 so it can be used by putchar
    BL   	PutChar                     ; send r0 to terminal
	POP 	{R0}
	;increment address pointer
	ADDS   	R0,R0,#1		

    ; loop
	B 	PutStringSB_PutNextChar
	
PutStringSB_End 
	POP	{R0-R2,PC}
        ENDP

;********************************************************************

PutNumU		PROC	{R0-R13}
; Description: takes in a number from r0, and outputs the decimal representation of that number
; Inputs:
;   	R0 number to print in decimal
; Outputs:
;   	none
; Registers modified:
; 	none
; Intermediate Registers
;	R0	Divisor / quotient
;	R1	Dividend / remainder
;	R2	pointer for PutNumUTempString
;	R3	address of PutNumUTempString


; todo add overflow protection

;	R1 = Temp decimal Digit	(remainder )
	PUSH 	{R0-R3,LR}
	LDR		R2,=PutNumUTempString	; address of PutNumUTempString
	MOVS	R1,R0			; move the number to print in decimal into r1
PutNumU_DivLoop				; R0 has dividend for divu
	MOVS	R0,#10		; move 10 into divisor for divu
	BL		DIVU		; after divu, the digit is in R1

	STRB	R1,[R2,#0]	;
	ADDS	R2,R2,#1
	CMP		R0,#0
	MOVS	R1,R0		; move the result (quotient) into the next number to be divided (dividend)
	BNE		PutNumU_DivLoop

; if we get here the decimal number is stored BACKWARDS as numbers (not ascii) in PutNumUTempString

	LDR	R3,=PutNumUTempString	; address of PutNumUTempString
PutNumU_PrintLoop
	SUBS	R2,R2,#1
	LDRB	R0,[R2,#0]
	ADDS	R0,R0,#48		;add the ascii offset for numbers '0'=48
	BL 		PutChar			; send the ascii char to terminal
	CMP		R2,R3			;compare the current pointer to the start of the temp string
	BHI		PutNumU_PrintLoop	; since the subtraction of the index occurs before this, loop as long as the Index is highere or equal to the starting address of the temp string

	POP     {R0-R3,PC}	;why is this causing an error??? cannot access memory
	ENDP


DIVU     PROC  {R2-R14}
;****************************************************************
;compute unsigned integer division: R0 remainder R1 = R1 ÷ R0.
;If the subroutine is called with R0 = 0, it should leave the input parameters unchanged and set the C flag on return.
; Otherwise, the C flag should be cleared on return.
;
; Input parameters:
;   R0: divisor (unsigned word by which to divide)
;   R1: dividend  (unsigned word to divide)

; Intermediate Values
; 	R2: quotient

;Output parameters:
;   R0:  quotient (unsigned word result of integer division)
;   R1:  remainder (unsigned word remainder from integer division)   C: APSR flag:  0 for valid result; 1 for invalid result

;****************************************************************
			PUSH	{R2}			; saving value of R2
			MOVS	R2,#0			; init R2

			; make sure the input is valid, no div by zero -------
            CMP     R0,#0           ; Compare R0 to zero
            BEQ     DivByZero       ; if the compare is equal, R0 contains zero, branch to DivByZero

			CMP		R1,#0			;COMPARE TO SEE IF DIVIDEND IS ZERO
			BEQ		DividendZero		;if that the case, branch to DividendZero
            ;input is valid, start the actual division ------------
startDivision
            CMP     R1,R0           ; Compare dividend - divisor
            ;got rid of this as we are chicking this cased with BLO;    BHS     divisionLoop    ;  Dividend is greater than divisor. This means division must occur, so branch to division loop
            ; if we get here, division has ended.
                ; The remainder is already set as it is what remains of the dividend. branch to end the subroutine

			BLO       ClearCFlag
divisionLoop
            SUBS    R1,R1,R0
            ADDS    R2,R2,#1
            B       startDivision


DivByZero
            ; save R0 and R1 to the stack
            PUSH    {R0-R1}
            ; set the C flag
            MRS   R0,APSR           ;move the APSR (special register) into R0
            LDR   R1,=APSR_C_MASK   ; Move the APSR_C_MASK into R1 (APSR_C_MASK = 00100000'00000000'00000000'00000000)
            ORRS  R0,R0,R1          ; logically OR the APSR_C_MASK (R1) with the APSR (R0) to preserver other flags while setting C flag
            MSR   APSR,R0
            ; load r0 and r1 back from the stack
            POP     {R0-R1}
            B     EndDIVU
DividendZero
		; since we are dividing zero by somthing, the answer will always be zero R 0
		MOVS	R0,#0
		MOVS	R1,#0

ClearCFlag
            ; clear C flag -------------
            PUSH    {R0-R1}
            ; set the C flag
            MRS     R0,APSR           ;move the APSR (special register) into R0
            LDR     R1,=APSR_C_MASK   ; ASK WHAT THIS IS DOING WITH PC;; Move the APSR_C_MASK into R1 (APSR_C_MASK = 00100000'00000000'00000000'00000000)
            BICS    R0,R0,R1            ; R0 gets R0 & (~APSR_C_MASK). so since (~APSR_C_MASK) is all ones except where the C flag is, r0 gets the contents of r1 with the C flag cleared
            MSR     APSR,R0             ;
            ; load r0 and r1 back from the stack
			POP     {R0-R1}
			MOVS	R0,R2				;Moved quotient into R0 for final output

EndDIVU

			POP		{R2}
            BX      LR
            ENDP
;--------------------------------

;-----------------------------------------------------------------------

;>>>>>   end subroutine code <<<<<

	END 